@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: downloader-view.c,v 1.79 2004/11/02 20:56:55 chpe Exp $
+ *  $Id: downloader-view.c,v 1.84.2.2 2005/04/08 20:03:47 chpe Exp $
  */
 
 #include "config.h" 
@@ -25,6 +25,8 @@
 #include "ephy-file-helpers.h"
 #include "ephy-embed-shell.h"
 #include "ephy-stock-icons.h"
+#include "ephy-gui.h"
+#include "ephy-debug.h"
 
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <eggstatusicon.h>
@@ -39,6 +41,10 @@
 #include <gtk/gtktreeviewcolumn.h>
 #include <gtk/gtkicontheme.h>
 #include <gtk/gtkiconfactory.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkmenushell.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtkmain.h>
 #include <libgnomeui/gnome-icon-lookup.h>
 #include <glib/gi18n.h>
 
@@ -157,11 +163,34 @@ downloader_view_class_init (DownloaderViewClass *klass)
 }
 
 static void
-status_icon_activated (EggStatusIcon *icon, DownloaderView *dv)
+show_downloader_cb (DownloaderView *dv)
 {
 	gtk_window_present (GTK_WINDOW (dv->priv->window));
 }
 
+static gboolean
+status_icon_popup_menu_cb (EggStatusIcon *icon,
+                           DownloaderView *dv)
+{
+        GtkWidget *menu, *item;
+	guint button = 0;
+
+	menu = gtk_menu_new ();
+
+	/* this opens the downloader window, or brings it to the foreground if already open */
+	item = gtk_menu_item_new_with_mnemonic (_("_Show Downloader..."));
+	g_signal_connect_swapped (item, "activate",
+				  G_CALLBACK (show_downloader_cb), dv);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show_all (menu);
+
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+			ephy_gui_menu_position_on_panel, icon->tray_icon,
+			button, gtk_get_current_event_time ());
+
+	return TRUE;
+}
+
 static void
 show_status_icon (DownloaderView *dv)
 {
@@ -171,8 +200,10 @@ show_status_icon (DownloaderView *dv)
 	dv->priv->status_icon = egg_status_icon_new_from_pixbuf (pixbuf);
 	g_object_unref (pixbuf);
 
-	g_signal_connect (dv->priv->status_icon, "activate",
-			  G_CALLBACK (status_icon_activated), dv);
+	g_signal_connect_swapped (dv->priv->status_icon, "activate",
+				  G_CALLBACK (show_downloader_cb), dv);
+	g_signal_connect (dv->priv->status_icon, "popup-menu",
+			  G_CALLBACK(status_icon_popup_menu_cb), dv);
 }
 
 static void
@@ -214,16 +245,17 @@ downloader_view_new (void)
 }
 
 static char *
-format_interval (long interval)
+format_interval (gint64 interval)
 {
-	int secs, hours, mins;														     
-	secs = (int)(interval + .5);
-	hours = secs / 3600;
-	secs -= hours * 3600;
-	mins = secs / 60;
-	secs -= mins * 60;
+	int hours, mins, secs;
 
-	if (hours)
+	hours = (int) interval / 3600;
+	interval -= hours * 3600;
+	mins = (int) interval / 60;
+	interval -= mins * 60;
+	secs = (int) interval;
+
+	if (hours > 0)
 	{
 		return g_strdup_printf (_("%u:%02u.%02u"), hours, mins, secs);
 	}
@@ -290,7 +322,7 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	EphyDownloadState state;
-	long total, current, remaining_secs = 0;
+	gint64 remaining_secs = 0, total, current;
 	char *remaining, *file, *cur_progress, *name;
 	struct tm;
 	int percent = 0;
@@ -326,7 +358,8 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 		char *total_progress;
 
 		total_progress = gnome_vfs_format_file_size_for_display (total);
-		file = g_strdup_printf ("%s\n%s of %s", name,
+		/* translators: first %s is filename, "%s of %s" is current/total file size */
+		file = g_strdup_printf (_("%s\n%s of %s"), name,
 					cur_progress, total_progress);
 		g_free (total_progress);
 	}
@@ -484,15 +517,21 @@ progress_cell_data_func (GtkTreeViewColumn *col,
 	switch (state)
 	{
 		case EPHY_DOWNLOAD_INITIALISING:
+			/* Translators: The text before the "|" is context to help you decide on
+			 * the correct translation. You MUST OMIT it in the translated string. */
 			text = Q_("download status|Unknown");
 			break;
 		case EPHY_DOWNLOAD_FAILED:
+			/* Translators: The text before the "|" is context to help you decide on
+			 * the correct translation. You MUST OMIT it in the translated string. */
 			text = Q_("download status|Failed");
 			break;
 		case EPHY_DOWNLOAD_DOWNLOADING:
 		case EPHY_DOWNLOAD_PAUSED:
 			if (percent == -1)
 			{
+				/* Translators: The text before the "|" is context to help you decide on
+				 * the correct translation. You MUST OMIT it in the translated string. */
 				text = Q_("download status|Unknown");
 				percent = 0;
 			}
@@ -589,6 +628,8 @@ downloader_view_build_ui (DownloaderView *dv)
 					   REMAINING_COL_POS);
 	gtk_tree_view_column_set_sort_column_id (column, COL_REMAINING);
 
+	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (priv->treeview), FALSE);
+
 	priv->model = GTK_TREE_MODEL (liststore);
 
 	gtk_window_set_icon_name (GTK_WINDOW (priv->window), EPHY_STOCK_DOWNLOAD);
