@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-single.h,v 1.28 2005/01/11 19:15:26 chpe Exp $
+ *  $Id: ephy-embed-single.h,v 1.30 2005/02/14 20:57:07 chpe Exp $
  */
 
 #ifndef EPHY_EMBED_SINGLE_H
@@ -53,6 +53,10 @@ struct _EphyEmbedSingleIface
 
 	/* Signals */
 
+	EphyEmbed * (* new_window)  (EphyEmbedSingle *single,
+				     EphyEmbed *parent_embed,
+				     EphyEmbedChrome chromemask);
+
 	gboolean (* handle_content) (EphyEmbedSingle *shell,
 				     char *mime_type,
 				     char *uri);
@@ -62,8 +66,13 @@ struct _EphyEmbedSingleIface
 
 
         gboolean (* add_sidebar)    (EphyEmbedSingle *single,
-				     const char * url,
-				     const char * title);
+				     const char *url,
+				     const char *title);
+
+        gboolean (* add_search_engine) (EphyEmbedSingle *single,
+					const char *url,
+					const char *icon_url,
+					const char *title);
 
 	gboolean (* check_content)  (EphyEmbedSingle *single,
 				     EphyContentCheckType type,
