@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-password-manager.h,v 1.6 2005/01/08 19:41:46 chpe Exp $
+ *  $Id: ephy-password-manager.h,v 1.7 2005/01/15 20:26:36 jframeau Exp $
  */
 
 #ifndef EPHY_PASSWORD_MANAGER_H
@@ -77,13 +77,13 @@ void			ephy_password_info_free		(EphyPasswordInfo *info);
 
 GType 		ephy_password_manager_get_type	(void);
 
-void		ephy_password_manager_add		(EphyPasswordManager *manager,
+void		ephy_password_manager_add_password	(EphyPasswordManager *manager,
 							 EphyPasswordInfo *info);
 
-void		ephy_password_manager_remove		(EphyPasswordManager *manager,
+void		ephy_password_manager_remove_password	(EphyPasswordManager *manager,
 							 EphyPasswordInfo *info);
 
-GList *		ephy_password_manager_list		(EphyPasswordManager *manager);
+GList *		ephy_password_manager_list_passwords	(EphyPasswordManager *manager);
 
 G_END_DECLS
 
