@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.h,v 1.27 2005/02/26 22:01:48 chpe Exp $
+ *  $Id: ContentHandler.h,v 1.27.2.1 2005/04/07 11:46:32 chpe Exp $
  */
 
 #ifndef CONTENT_HANDLER_H
@@ -28,9 +28,9 @@
 
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
-#define MOZILLA_STRICT_API
+#undef MOZILLA_INTERNAL_API
 #include <nsEmbedString.h>
-#undef MOZILLA_STRICT_API
+#define MOZILLA_INTERNAL_API 1
 #include <nsCOMPtr.h>
 #include <nsISupports.h>
 #include <nsIURI.h>
