@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.49 2005/02/02 19:48:10 chpe Exp $
+ *  $Id: EventContext.cpp,v 1.50.2.1 2005/04/07 11:46:32 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -31,9 +31,9 @@
 
 #include <nsIInterfaceRequestor.h>
 #include <nsIServiceManager.h>
-#define MOZILLA_STRICT_API
+#undef MOZILLA_INTERNAL_API
 #include <nsEmbedString.h>
-#undef MOZILLA_STRICT_API
+#define MOZILLA_INTERNAL_API 1
 #include <nsIDOMEventTarget.h>
 #include <nsIDOMHTMLInputElement.h>
 #include <nsIDOMHTMLObjectElement.h>
@@ -813,6 +813,11 @@ nsresult EventContext::CheckInput (nsIDOMNode *aNode)
 		 g_ascii_strcasecmp (value.get(), "checkbox") != 0)
 	{
 		mEmbedEvent->context |= EPHY_EMBED_CONTEXT_INPUT;
+
+		if (g_ascii_strcasecmp (value.get(), "password") == 0)
+		{
+			mEmbedEvent->context |= EPHY_EMBED_CONTEXT_INPUT_PASSWORD;
+		}
 	}
 
 	return NS_OK;
