@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-extensions-manager.c,v 1.18 2004/11/18 15:18:55 chpe Exp $
+ *  $Id: ephy-extensions-manager.c,v 1.24 2005/01/28 22:24:45 chpe Exp $
  */
 
 #include "config.h"
@@ -39,6 +39,8 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
+#include <gconf/gconf-client.h>
+
 #include <gmodule.h>
 #include <dirent.h>
 #include <string.h>
@@ -165,7 +167,7 @@ ephy_extensions_manager_load (EphyExtensionsManager *manager,
 	g_return_if_fail (EPHY_IS_EXTENSIONS_MANAGER (manager));
 	g_return_if_fail (identifier != NULL);
 
-	LOG ("Adding '%s' to extensions", identifier)
+	LOG ("Adding '%s' to extensions", identifier);
 
 	gconf_exts = eel_gconf_get_string_list (CONF_LOADED_EXTENSIONS);
 
@@ -201,7 +203,7 @@ ephy_extensions_manager_unload (EphyExtensionsManager *manager,
 	g_return_if_fail (EPHY_IS_EXTENSIONS_MANAGER (manager));
 	g_return_if_fail (identifier != NULL);
 
-	LOG ("Removing '%s' from extensions", identifier)
+	LOG ("Removing '%s' from extensions", identifier);
 
 	gconf_exts = eel_gconf_get_string_list (CONF_LOADED_EXTENSIONS);
 
@@ -369,7 +371,7 @@ ephy_extensions_manager_load_string (EphyExtensionsManager *manager,
 	LocalisedString description = { NULL, G_MAXUINT };
 	LocalisedString name = { NULL, G_MAXUINT };
 
-	LOG ("Loading description file for '%s'", identifier)
+	LOG ("Loading description file for '%s'", identifier);
 
 	if (g_list_find_custom (manager->priv->data, identifier,
 				(GCompareFunc) find_extension_info) != NULL)
@@ -705,14 +707,32 @@ find_loader (const LoaderInfo *info,
 	return strcmp (info->type, type);
 }
 
+static char *
+sanitise_type (const char *string)
+{
+	char *str, *p;
+
+	str = g_strdup (string);
+	for (p = str; *p != '\0'; p++)
+	{
+		if (!g_ascii_isalpha (*p)) *p = '-';
+	}
+
+	return str;
+}
+
 static EphyLoader *
 get_loader_for_type (EphyExtensionsManager *manager,
-		      const char *type)
+		     const char *type)
 {
 	LoaderInfo *info;
 	GList *l;
+	GData *attr = NULL;
+	char *path, *name, *stype;
+	EphyLoader *shlib_loader;
+	GObject *loader;
 
-	LOG ("Looking for loader for type '%s'", type)
+	LOG ("Looking for loader for type '%s'", type);
 
 	l = g_list_find_custom (manager->priv->factories, type,
 				(GCompareFunc) find_loader);
@@ -734,18 +754,53 @@ get_loader_for_type (EphyExtensionsManager *manager,
 		return g_object_ref (info->loader);
 	}
 
-	/* try to load a loader */
+	stype = sanitise_type (type);
+	name = g_strconcat ("lib", stype, "loader.", G_MODULE_SUFFIX, NULL);
+	path = g_build_filename (LOADER_DIR, name, NULL);
+	g_datalist_init (&attr);
+	g_datalist_set_data (&attr, "library", path);
+
+	shlib_loader = get_loader_for_type (manager, "shlib");
+	g_return_val_if_fail (shlib_loader != NULL, NULL);
+
+	loader = ephy_loader_get_object (shlib_loader, &attr);
+	g_datalist_clear (&attr);
+	g_free (stype);
+	g_free (name);
+	g_free (path);
+
+	if (EPHY_IS_LOADER (loader))
+	{
+		info = g_new (LoaderInfo, 1);
+		info->type = g_strdup (type);
+		info->loader = EPHY_LOADER (loader);
+
+		manager->priv->factories =
+			g_list_append (manager->priv->factories, info);
+
+		return g_object_ref (info->loader);
+	}
+
 	g_return_val_if_reached (NULL);
 
 	return NULL;
 }
 
-
 static void
 attach_window (EphyWindow *window,
 	       EphyExtension *extension)
 {
+	GList *tabs, *l;
+
 	ephy_extension_attach_window (extension, window);
+
+	tabs = ephy_window_get_tabs (window);
+	for (l = tabs; l; l = l->next)
+	{
+		ephy_extension_attach_tab (extension, window,
+					   EPHY_TAB (l->data));
+	}
+	g_list_free (tabs);
 }
 
 static void
@@ -756,7 +811,7 @@ load_extension (EphyExtensionsManager *manager,
 
 	g_return_if_fail (info->extension == NULL);
 
-	LOG ("Loading extension '%s'", info->info.identifier)
+	LOG ("Loading extension '%s'", info->info.identifier);
 
 	/* don't try again */
 	if (info->load_failed) return;
@@ -803,6 +858,16 @@ static void
 detach_window (EphyWindow *window,
 	       EphyExtension *extension)
 {
+	GList *tabs, *l;
+
+	tabs = ephy_window_get_tabs (window);
+	for (l = tabs; l; l = l->next)
+	{
+		ephy_extension_detach_tab (extension, window,
+					   EPHY_TAB (l->data));
+	}
+	g_list_free (tabs);
+
 	ephy_extension_detach_window (extension, window);
 }
 
@@ -821,7 +886,7 @@ unload_extension (EphyExtensionsManager *manager,
 	g_return_if_fail (info->loader != NULL);
 	g_return_if_fail (info->extension != NULL || info->load_failed);
 
-	LOG ("Unloading extension '%s'", info->info.identifier)
+	LOG ("Unloading extension '%s'", info->info.identifier);
 
 	if (info->load_failed) return;
 
@@ -833,15 +898,15 @@ unload_extension (EphyExtensionsManager *manager,
 
 		manager->priv->extensions =
 			g_list_remove (manager->priv->extensions, info->extension);
-	}
 
-	/* we own two refs to the extension, the one we added when
-	 * we added it to the priv->extensions list, and the one returned
-	 * from get_object. Release object, and queue a unref, since if the
-	 * extension has its own functions queued in the idle loop, the
-	 * functions must exist in memory before being called.
-	 */
-	g_idle_add ((GSourceFunc) idle_unref, info->extension);
+		/* we own two refs to the extension, the one we added when
+		 * we added it to the priv->extensions list, and the one returned
+		 * from get_object. Release object, and queue a unref, since if the
+		 * extension has its own functions queued in the idle loop, the
+		 * functions must exist in memory before being called.
+		 */
+		g_idle_add ((GSourceFunc) idle_unref, info->extension);
+	}
 
 	ephy_loader_release_object (info->loader, G_OBJECT (info->extension));
 
@@ -854,14 +919,33 @@ unload_extension (EphyExtensionsManager *manager,
 static void
 sync_loaded_extensions (EphyExtensionsManager *manager)
 {
-	GSList *active_extensions;
+	GConfClient *client;
+	GConfValue *value;
+	GSList *active_extensions = NULL;
 	GList *l;
 	gboolean active;
 	ExtensionInfo *info;
 
-	LOG ("Synching changed list of active extensions")
+	LOG ("Synching changed list of active extensions");
 
-	active_extensions = eel_gconf_get_string_list (CONF_LOADED_EXTENSIONS);
+	client = gconf_client_get_default ();
+	g_return_if_fail (client != NULL);
+
+	value = gconf_client_get (client, CONF_LOADED_EXTENSIONS, NULL);
+
+	/* make sure the extensions-manager-ui is loaded */
+	if (value == NULL ||
+	    value->type != GCONF_VALUE_LIST ||
+	    gconf_value_get_list_type (value) != GCONF_VALUE_STRING)
+	{
+		active_extensions = g_slist_prepend (active_extensions,
+						     g_strdup ("extensions-manager-ui"));
+		eel_gconf_set_string_list (CONF_LOADED_EXTENSIONS, active_extensions);
+	}
+	else
+	{
+		active_extensions = eel_gconf_get_string_list (CONF_LOADED_EXTENSIONS);
+	}
 
 	for (l = manager->priv->data; l != NULL; l = l->next)
 	{
@@ -874,7 +958,7 @@ sync_loaded_extensions (EphyExtensionsManager *manager)
 		LOG ("Extension '%s' is %sactive and %sloaded",
 		     info->info.identifier,
 		     active ? "" : "not ",
-		     info->info.active ? "" : "not ")
+		     info->info.active ? "" : "not ");
 
 		if (active != info->info.active)
 		{
@@ -891,6 +975,12 @@ sync_loaded_extensions (EphyExtensionsManager *manager)
 
 	g_slist_foreach (active_extensions, (GFunc) g_free, NULL);
 	g_slist_free (active_extensions);
+
+	if (value != NULL)
+	{
+		gconf_value_free (value);
+	}
+	g_object_unref (client);
 }
 
 static void
@@ -1022,7 +1112,7 @@ ephy_extensions_manager_load_dir (EphyExtensionsManager *manager,
 	GnomeVFSMonitorHandle *monitor;
 	GnomeVFSResult res;
 
-	LOG ("Scanning directory '%s'", path)
+	LOG ("Scanning directory '%s'", path);
 
 	START_PROFILER ("Scanning directory")
 
@@ -1132,7 +1222,7 @@ ephy_extensions_manager_init (EphyExtensionsManager *manager)
 {
 	manager->priv = EPHY_EXTENSIONS_MANAGER_GET_PRIVATE (manager);
 
-	LOG ("EphyExtensionsManager initialising")
+	LOG ("EphyExtensionsManager initialising");
 }
 
 void
@@ -1142,7 +1232,7 @@ ephy_extensions_manager_startup (EphyExtensionsManager *manager)
 
 	g_return_if_fail (EPHY_IS_EXTENSIONS_MANAGER (manager));
 
-	LOG ("EphyExtensionsManager startup")
+	LOG ("EphyExtensionsManager startup");
 
 #ifdef ENABLE_VALIDATION
 	init_schema_ctxt (manager);
@@ -1169,7 +1259,7 @@ ephy_extensions_manager_finalize (GObject *object)
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (object);
 	EphyExtensionsManagerPrivate *priv = manager->priv;
 
-	LOG ("EphyExtensionsManager finalising")
+	LOG ("EphyExtensionsManager finalising");
 
 	eel_gconf_notification_remove (manager->priv->active_extensions_notifier_id);
 
@@ -1179,6 +1269,7 @@ ephy_extensions_manager_finalize (GObject *object)
 	g_list_foreach (priv->extensions, (GFunc) g_object_unref, NULL);
 	g_list_free (priv->extensions);
 
+	/* FIXME release loaded loaders */
 	g_list_foreach (priv->factories, (GFunc) free_loader_info, NULL);
 	g_list_free (priv->factories);
 
@@ -1201,16 +1292,23 @@ ephy_extensions_manager_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
+static void
+attach_extension_to_window (EphyExtension *extension,
+			    EphyWindow *window)
+{
+	attach_window (window, extension);
+}
+
 static void
 impl_attach_window (EphyExtension *extension,
 		    EphyWindow *window)
 {
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (extension);
 
-	LOG ("Attach")
+	LOG ("Attach");
 
 	g_list_foreach (manager->priv->extensions,
-			(GFunc) ephy_extension_attach_window, window);
+			(GFunc) attach_extension_to_window, window);
 
 	manager->priv->windows = g_list_prepend (manager->priv->windows, window);
 }
@@ -1220,24 +1318,77 @@ impl_detach_window (EphyExtension *extension,
 		    EphyWindow *window)
 {
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (extension);
+	GList *tabs, *l;
 
-	LOG ("Detach")
+	LOG ("Detach");
 
 	manager->priv->windows = g_list_remove (manager->priv->windows, window);
 
 	g_object_ref (window);
 
+	/* Detach tabs (uses impl_detach_tab) */
+	tabs = ephy_window_get_tabs (window);
+	for (l = tabs; l; l = l->next)
+	{
+		ephy_extension_detach_tab (extension, window,
+					   EPHY_TAB (l->data));
+	}
+	g_list_free (tabs);
+
+	/* Then detach the window */
 	g_list_foreach (manager->priv->extensions,
 			(GFunc) ephy_extension_detach_window, window);
 
 	g_object_unref (window);
 }
 
+static void
+impl_attach_tab (EphyExtension *extension,
+		 EphyWindow *window,
+		 EphyTab *tab)
+{
+	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (extension);
+	GList *l;
+
+	LOG ("Attach tab");
+
+	for (l = manager->priv->extensions; l; l = l->next)
+	{
+		ephy_extension_attach_tab (EPHY_EXTENSION (l->data),
+					   window, tab);
+	}
+}
+
+static void
+impl_detach_tab (EphyExtension *extension,
+		 EphyWindow *window,
+		 EphyTab *tab)
+{
+	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (extension);
+	GList *l;
+
+	LOG ("Detach Tab");
+
+	g_object_ref (window);
+	g_object_ref (tab);
+
+	for (l = manager->priv->extensions; l; l = l->next)
+	{
+		ephy_extension_detach_tab (EPHY_EXTENSION (l->data),
+					   window, tab);
+	}
+
+	g_object_unref (tab);
+	g_object_unref (window);
+}
+
 static void
 ephy_extensions_manager_iface_init (EphyExtensionIface *iface)
 {
 	iface->attach_window = impl_attach_window;
 	iface->detach_window = impl_detach_window;
+	iface->attach_tab    = impl_attach_tab;
+	iface->detach_tab    = impl_detach_tab;
 }
 
 static void
