@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2000-2003 Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2003, 2004, 2005 Christian Persch
  *  Copyright (C) 2004 Crispin Flowerday
  *  Copyright (C) 2004 Adam Hooper
  *
@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.135 2005/01/23 15:38:02 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.139 2005/02/24 15:00:17 chpe Exp $
  */
 
 #include "config.h"
@@ -42,6 +42,7 @@
 #include "ephy-embed-single.h"
 #include "ephy-shell.h"
 #include "ephy-permission-manager.h"
+#include "ephy-link.h"
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -57,14 +58,11 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkuimanager.h>
-#include <gtk/gtkradioaction.h>
 #include <gtk/gtkclipboard.h>
 #include <string.h>
 
 #define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
 
-#define CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME  "/apps/epiphany/lockdown/disable_javascript_chrome"
-
 #define MAX_HIDDEN_POPUPS 5
 
 struct _EphyTabPrivate
@@ -82,7 +80,6 @@ struct _EphyTabPrivate
 	int total_requests;
 	int width;
 	int height;
-	GtkToggleAction *action;
 	float zoom;
 	gboolean setting_zoom;
 	EphyEmbedSecurityLevel security_level;
@@ -122,8 +119,6 @@ typedef struct
 
 static GObjectClass *parent_class = NULL;
 
-static gulong tab_id = 0;
-
 /* internal functions, accessible only from this file */
 static void	ephy_tab_set_icon_address	(EphyTab *tab,
 						 const char *location);
@@ -139,7 +134,7 @@ static void	ephy_tab_set_security_level	(EphyTab *tab,
 						 EphyEmbedSecurityLevel level);
 static void	ephy_tab_set_title		(EphyTab *tab,
 						 EphyEmbed *embed,
-						 const char *new_title);
+						 char *new_title);
 static void	ephy_tab_set_zoom		(EphyTab *tab,
 						 float zoom);
 static guint	popup_blocker_n_hidden		(EphyTab *tab);
@@ -168,10 +163,21 @@ ephy_tab_get_type (void)
                         0, /* n_preallocs */
                         (GInstanceInitFunc) ephy_tab_init
                 };
+		static const GInterfaceInfo link_info = 
+		{
+			NULL,
+			NULL,
+			NULL
+		};
+
 
                 type = g_type_register_static (GTK_TYPE_BIN,
 					       "EphyTab",
 					       &our_info, 0);
+
+		g_type_add_interface_static (type,
+					     EPHY_TYPE_LINK,
+					     &link_info);
         }
 
         return type;
@@ -329,23 +335,6 @@ ephy_tab_get_window (EphyTab *tab)
 	return EPHY_WINDOW (toplevel);
 }
 
-static void
-ephy_tab_action_activate_cb (GtkAction *action, EphyTab *tab)
-{
-	EphyWindow *window;
-
-	g_return_if_fail (EPHY_IS_TAB (tab));
-
-	window = ephy_tab_get_window (tab);
-	g_return_if_fail (window != NULL);
-
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) &&
-	    ephy_window_get_active_tab (window) != tab)
-	{
-		ephy_window_jump_to_tab (window, tab);
-	}
-}
-
 static void
 ephy_tab_class_init (EphyTabClass *class)
 {
@@ -494,7 +483,7 @@ popups_manager_add (EphyTab *tab,
 	PopupInfo *popup;
 
 	LOG ("popups_manager_add: tab %p, url %s, features %s",
-	     tab, url, features)
+	     tab, url, features);
 
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
@@ -549,7 +538,7 @@ static void
 popups_manager_add_window (EphyTab *tab,
 			   EphyWindow *window)
 {
-	LOG ("popups_manager_add_window: tab %p, window %p", tab, window)
+	LOG ("popups_manager_add_window: tab %p, window %p", tab, window);
 
 	g_return_if_fail (EPHY_IS_TAB (tab));
 	g_return_if_fail (EPHY_IS_WINDOW (window));
@@ -604,7 +593,7 @@ ephy_tab_get_popups_allowed (EphyTab *tab)
 
 	g_free (location);
 
-	LOG ("ephy_tab_get_popups_allowed: tab %p, allowed: %d", tab, allow)
+	LOG ("ephy_tab_get_popups_allowed: tab %p, allowed: %d", tab, allow);
 
 	return allow;
 }
@@ -630,7 +619,7 @@ popups_manager_show (PopupInfo *popup,
 static void
 popups_manager_show_all (EphyTab *tab)
 {
-	LOG ("popup_blocker_show_all: tab %p", tab)
+	LOG ("popup_blocker_show_all: tab %p", tab);
 
 	g_slist_foreach (tab->priv->hidden_popups,
 			 (GFunc) popups_manager_show, tab);
@@ -687,7 +676,7 @@ popups_manager_hide (EphyWindow *window,
 static void
 popups_manager_hide_all (EphyTab *tab)
 {
-	LOG ("popup_blocker_hide_all: tab %p", tab)
+	LOG ("popup_blocker_hide_all: tab %p", tab);
 
 	g_slist_foreach (tab->priv->shown_popups,
 			 (GFunc) popups_manager_hide, tab);
@@ -763,11 +752,6 @@ ephy_tab_finalize (GObject *object)
 
 	g_idle_remove_by_data (tab);
 
-	if (tab->priv->action != NULL)
-	{
-		g_object_unref (tab->priv->action);
-	}
-
 	g_free (tab->priv->title);
 	g_free (tab->priv->address);
 	g_free (tab->priv->icon_address);
@@ -777,7 +761,7 @@ ephy_tab_finalize (GObject *object)
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 
-	LOG ("EphyTab finalized %p", tab)
+	LOG ("EphyTab finalized %p", tab);
 }
 
 static gboolean
@@ -1049,7 +1033,7 @@ ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 {
 	const char *uv_address;
 
-	LOG ("ephy_tab_address_cb tab %p address %s", tab, address)
+	LOG ("ephy_tab_address_cb tab %p address %s", tab, address);
 
 	/* Do not expose about:blank to the user, an empty address
 	   bar will do better */
@@ -1125,7 +1109,7 @@ ephy_tab_zoom_changed_cb (EphyEmbed *embed, float zoom, EphyTab *tab)
 {
 	char *address;
 
-	LOG ("ephy_tab_zoom_changed_cb tab %p zoom %f", tab, zoom)
+	LOG ("ephy_tab_zoom_changed_cb tab %p zoom %f", tab, zoom);
 
 	ephy_tab_set_zoom (tab, zoom);
 
@@ -1169,8 +1153,6 @@ ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
 	title = ephy_embed_get_title (embed);
 
 	ephy_tab_set_title (tab, embed, title);
-
-	g_free (title);
 }
 
 static int
@@ -1282,13 +1264,13 @@ build_progress_from_requests (EphyTab *tab, EphyEmbedNetState state)
 static void
 ensure_page_info (EphyTab *tab, EphyEmbed *embed, const char *address)
 {
-	if ((tab->priv->address == NULL || *tab->priv->address == '\0') &&
+	if ((tab->priv->address == NULL || tab->priv->address[0] == '\0') &&
 	    tab->priv->address_expire == EPHY_TAB_ADDRESS_EXPIRE_NOW)
         {
 		ephy_tab_set_location (tab, address, EPHY_TAB_ADDRESS_EXPIRE_NOW);
 	}
 
-	if (tab->priv->title == NULL)
+	if (tab->priv->title == NULL || tab->priv->title[0] == '\0')
 	{
 		ephy_tab_set_title (tab, embed, NULL);
 	}
@@ -1326,34 +1308,19 @@ ephy_tab_net_state_cb (EphyEmbed *embed,
 	build_progress_from_requests (tab, state);
 }
 
-static EphyEmbed *
+static void
 ephy_tab_new_window_cb (EphyEmbed *embed,
-			EphyEmbedChrome chromemask,
+			EphyEmbed *new_embed,
 			EphyTab *tab)
 {
-	EphyTab *new_tab;
 	EphyWindow *window;
 
-	LOG ("ephy_tab_new_window_cb tab %p with parent %p chrome %d",
-	     tab, ((GtkWidget *) tab)->parent, chromemask)
+	g_return_if_fail (new_embed != NULL);
 
-	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME))
-	{
-		window = ephy_window_new ();
-	}
-	else
-	{
-		window = ephy_window_new_with_chrome (chromemask);
-	}
-
-	new_tab = ephy_tab_new ();
-	gtk_widget_show (GTK_WIDGET (new_tab));
-
-        ephy_window_add_tab (window, new_tab, -1, FALSE);
+	window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (new_embed)));
+	g_return_if_fail (window != NULL);
 
 	popups_manager_add_window (tab, window);
-
-	return ephy_tab_get_embed (new_tab);
 }
 
 static void
@@ -1376,7 +1343,7 @@ ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
 			EphyTab *tab)
 {
 	LOG ("ephy_tab_visibility_cb tab %p visibility %d",
-	     tab, visibility)
+	     tab, visibility);
 
 	if (visibility)
 	{
@@ -1401,7 +1368,7 @@ ephy_tab_destroy_brsr_cb (EphyEmbed *embed, EphyTab *tab)
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	LOG ("ephy_tab_destroy_browser_cb tab %p parent %p",
-	     tab, ((GtkWidget *) tab)->parent)
+	     tab, ((GtkWidget *) tab)->parent);
 
 	window = ephy_tab_get_window (tab);
 	g_return_if_fail (window != NULL);
@@ -1419,7 +1386,7 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 	GtkWidget *notebook;
 	EphyWindow *window;
 
-	LOG ("ephy_tab_size_to_cb tab %p width %d height %d", tab, width, height)
+	LOG ("ephy_tab_size_to_cb tab %p width %d height %d", tab, width, height);
 
 	tab->priv->width = width;
 	tab->priv->height = height;
@@ -1447,6 +1414,7 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 	}
 }
 
+
 static gboolean
 open_link_in_new_tab (EphyTab *tab,
 		      const char *link_address)
@@ -1461,13 +1429,11 @@ open_link_in_new_tab (EphyTab *tab,
 
 	if (new_tab)
 	{
-		ephy_shell_new_tab (ephy_shell, window, tab,
-				    link_address,
-				    EPHY_NEW_TAB_OPEN_PAGE |
-				    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+		return ephy_link_open (EPHY_LINK (tab), link_address,
+				       tab, EPHY_LINK_NEW_TAB) != NULL;
 	}
 
-	return new_tab;
+	return FALSE;
 }
 
 static void
@@ -1504,8 +1470,9 @@ clipboard_text_received_cb (GtkClipboard *clipboard,
 	if (*weak_ptr != NULL && text != NULL)
 	{
 		EphyEmbed *embed = (EphyEmbed *) *weak_ptr;
+		EphyTab *tab = ephy_tab_for_embed (embed);
 
-		ephy_embed_load_url (embed, text);
+		ephy_link_open (EPHY_LINK (tab), text, tab, 0);
 	}
 
 	if (*weak_ptr != NULL)
@@ -1536,7 +1503,7 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	modifier = ephy_embed_event_get_modifier (event);
 
 	LOG ("ephy_tab_dom_mouse_click_cb: button %d, context %x, modifier %x",
-	     button, context, modifier)
+	     button, context, modifier);
 
 	with_control = (modifier & GDK_CONTROL_MASK) != 0;
 	with_shift = (modifier & GDK_SHIFT_MASK) != 0;
@@ -1617,9 +1584,8 @@ ephy_tab_init (EphyTab *tab)
 {
 	GObject *embed;
 	EphyFaviconCache *cache;
-	char *id;
 
-	LOG ("EphyTab initialising %p", tab)
+	LOG ("EphyTab initialising %p", tab);
 
 	tab->priv = EPHY_TAB_GET_PRIVATE (tab);
 
@@ -1633,6 +1599,7 @@ ephy_tab_init (EphyTab *tab)
 	tab->priv->document_type = EPHY_EMBED_DOCUMENT_HTML;
 	tab->priv->zoom = 1.0;
 	tab->priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
+	tab->priv->title = g_strdup ("");
 
 	embed = ephy_embed_factory_new_object (EPHY_TYPE_EMBED);
 	g_assert (embed != NULL);
@@ -1640,18 +1607,6 @@ ephy_tab_init (EphyTab *tab)
 	gtk_container_add (GTK_CONTAINER (tab), GTK_WIDGET (embed));
 	gtk_widget_show (GTK_WIDGET (embed));
 
-	id = g_strdup_printf ("Tab%lu", tab_id++);
-
-	tab->priv->action = g_object_new (GTK_TYPE_TOGGLE_ACTION,
-					  "name", id,
-					  "label", _("Blank page"),
-					  "draw_as_radio", TRUE,
-					  NULL);
-
-	g_free (id);
-	g_signal_connect (tab->priv->action, "activate",
-			  G_CALLBACK (ephy_tab_action_activate_cb), tab);
-
 	g_signal_connect_object (embed, "link_message",
 				 G_CALLBACK (ephy_tab_link_message_cb),
 				 tab, 0);
@@ -1820,30 +1775,27 @@ ephy_tab_get_status_message (EphyTab *tab)
 	}
 }
 
-#define MAX_LABEL_LENGTH	32
-
 static void
-ephy_tab_set_title (EphyTab *tab, EphyEmbed *embed, const char *new_title)
+ephy_tab_set_title (EphyTab *tab,
+		    EphyEmbed *embed,
+		    char *title)
 {
-	char *title = NULL;
-
-	g_return_if_fail (EPHY_IS_TAB (tab));
-
-	g_free (tab->priv->title);
-
-	if (new_title == NULL || new_title[0] == '\0')
+ 	if (title == NULL || title[0] == '\0')
 	{
 		GnomeVFSURI *uri = NULL;
 		char *address;
 
+		g_free (title);
+		title = NULL;
+
 		address = ephy_embed_get_location (embed, TRUE);
 
-		if (address)
+		if (address != NULL)
 		{
 			uri = gnome_vfs_uri_new (address);
 		}
 
-		if (uri)
+		if (uri != NULL)
 		{
 			title = gnome_vfs_uri_to_string (uri,
 					GNOME_VFS_URI_HIDE_USER_NAME |
@@ -1852,31 +1804,29 @@ ephy_tab_set_title (EphyTab *tab, EphyEmbed *embed, const char *new_title)
 					GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD |
 					GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
 			gnome_vfs_uri_unref (uri);
+			g_free (address);
 		}
-		else if (address != NULL && strncmp (address, "about:blank", 11) != 0)
+		else if (address != NULL &&
+			 strncmp (address, "about:blank", 11) != 0)
 		{
-			title = g_strdup (address);
+			title = address;
+		}
+		else
+		{
+			g_free (address);
 		}
 
+		/* Fallback */
 		if (title == NULL || title[0] == '\0')
 		{
 			g_free (title);
 			title = g_strdup (_("Blank page"));
 		}
-
-		g_free (address);
-	}
-	else
-	{
-		title = g_strdup (new_title);
 	}
 
+	g_free (tab->priv->title);
 	tab->priv->title = title;
 
-	g_object_set (G_OBJECT (tab->priv->action),
-		      "label", title,
-		      NULL);
-
 	g_object_notify (G_OBJECT (tab), "title");
 }
 
@@ -1891,7 +1841,7 @@ ephy_tab_set_title (EphyTab *tab, EphyEmbed *embed, const char *new_title)
 const char *
 ephy_tab_get_title (EphyTab *tab)
 {
-	g_return_val_if_fail (EPHY_IS_TAB (tab), "");
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	return tab->priv->title;
 }
@@ -2001,23 +1951,3 @@ ephy_tab_get_zoom (EphyTab *tab)
 
 	return tab->priv->zoom;
 }
-
-/**
- * ephy_tab_get_action:
- * @tab: an #EphyTab
- *
- * DO NOT USE
- *
- * Returns the #GtkToggleAction represented by the labelled tab displayed at the
- * top of @tab's #EphyWindow when multiple tabs are loaded. Activating this
- * action will switch the window to display @tab.
- *
- * Return value: @tab's #GtkToggleAction
- **/
-GObject *
-ephy_tab_get_action (EphyTab *tab)
-{
-	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
-
-	return G_OBJECT (tab->priv->action);
-}
